defmodule Archeometer.Analysis.Treemap.Data do
  @moduledoc false

  import Archeometer.Query
  alias Archeometer.Repo
  alias Archeometer.Schema.Module

  def get_data(:size, app, namespace, db_name, skip_tests) do
    get_modules_and_lcount(app, namespace, db_name, skip_tests)
  end

  defp get_modules_and_lcount(app, ns, db_name, skip_tests) do
    is_tests_vals = if skip_tests, do: [false], else: [true, false]
    modules = get_modules(app, ns, db_name, is_tests_vals)

    Enum.map(modules, fn [id, name, num_lines] -> {id, name, num_lines} end)
  end

  defp get_modules(:none, "*", db_name, is_tests_vals) do
    %{rows: modules} =
      Repo.all(
        from(m in Module,
          select: [
            id: m.id,
            name: m.name,
            num_lines: m.num_lines
          ],
          where: m.is_test in ^is_tests_vals
        ),
        [],
        db_name
      )

    modules
  end

  defp get_modules(:none, ns, db_name, is_tests_vals) do
    %{rows: modules} =
      Repo.all(
        from(m in Module,
          select: [
            id: m.id,
            name: m.name,
            num_lines: m.num_lines
          ],
          where:
            (m.name == ^ns or like(m.name, ^(ns <> ".%"))) and
              m.is_test in ^is_tests_vals
        ),
        [],
        db_name
      )

    modules
  end

  defp get_modules(app, "*", db_name, is_tests_vals) do
    app_str = to_string(app)

    %{rows: modules} =
      Repo.all(
        from(m in Module,
          select: [
            id: m.id,
            name: m.name,
            num_lines: m.num_lines
          ],
          where:
            m.application.name == ^app_str and
              m.is_test in ^is_tests_vals
        ),
        [],
        db_name
      )

    modules
  end

  defp get_modules(app, ns, db_name, is_tests_vals) do
    app_str = to_string(app)

    %{rows: modules} =
      Repo.all(
        from(m in Module,
          select: [
            id: m.id,
            name: m.name,
            num_lines: m.num_lines
          ],
          where:
            m.application.name == ^app_str and
              (m.name == ^ns or like(m.name, ^(ns <> ".%"))) and
              m.is_test in ^is_tests_vals
        ),
        [],
        db_name
      )

    modules
  end
end
